@@ -11,10 +11,12 @@ use caller::Caller;
 use counter::Counter;
 use counter_float::CounterFloat;
 use delegator::Delegator;
-use dusk_abi::{Module, Transaction};
+use dusk_abi::Transaction;
 use fibonacci::Fibonacci;
 use gas_consumed::GasConsumed;
-use rusk_vm::{Contract, ContractId, GasMeter, NetworkState};
+use rusk_vm::{
+    Contract, ContractId, GasMeter, NetworkState, Schedule, VMError,
+};
 use self_snapshot::SelfSnapshot;
 use tx_vec::TxVec;
 
@@ -35,7 +37,7 @@ fn counter() {
 
     let contract = Contract::new(counter, code.to_vec());
 
-    let mut network = NetworkState::default();
+    let mut network = NetworkState::new();
 
     let contract_id = network.deploy(contract).unwrap();
 
@@ -43,18 +45,18 @@ fn counter() {
 
     assert_eq!(
         network
-            .query::<_, i32>(contract_id, counter::READ_VALUE, &mut gas)
+            .query::<_, i32>(contract_id, 0, counter::READ_VALUE, &mut gas)
             .unwrap(),
         99
     );
 
     network
-        .transact::<_, ()>(contract_id, counter::INCREMENT, &mut gas)
+        .transact::<_, ()>(contract_id, 0, counter::INCREMENT, &mut gas)
         .unwrap();
 
     assert_eq!(
         network
-            .query::<_, i32>(contract_id, counter::READ_VALUE, &mut gas)
+            .query::<_, i32>(contract_id, 0, counter::READ_VALUE, &mut gas)
             .unwrap(),
         100
     );
@@ -69,7 +71,7 @@ fn counter_trivial() {
 
     let contract = Contract::new(counter, code.to_vec());
 
-    let mut network = NetworkState::default();
+    let mut network = NetworkState::new();
 
     let contract_id = network.deploy(contract).unwrap();
 
@@ -77,7 +79,7 @@ fn counter_trivial() {
 
     assert_eq!(
         network
-            .query::<_, i32>(contract_id, counter::READ_VALUE, &mut gas)
+            .query::<_, i32>(contract_id, 0, counter::READ_VALUE, &mut gas)
             .unwrap(),
         99
     );
@@ -88,7 +90,7 @@ fn delegated_call() {
     let counter = Counter::new(99);
     let delegator = Delegator;
 
-    let mut network = NetworkState::default();
+    let mut network = NetworkState::new();
 
     let counter_code =
         include_bytes!("../target/wasm32-unknown-unknown/release/counter.wasm");
@@ -110,6 +112,7 @@ fn delegated_call() {
         network
             .query::<_, i32>(
                 delegator_id,
+                0,
                 (delegator::DELEGATE_QUERY, counter_id, counter::READ_VALUE),
                 &mut gas
             )
@@ -122,6 +125,7 @@ fn delegated_call() {
     network
         .transact::<_, ()>(
             delegator_id,
+            0,
             (
                 delegator::DELEGATE_TRANSACTION,
                 counter_id,
@@ -135,7 +139,7 @@ fn delegated_call() {
 
     assert_eq!(
         network
-            .query::<_, i32>(counter_id, counter::READ_VALUE, &mut gas)
+            .query::<_, i32>(counter_id, 0, counter::READ_VALUE, &mut gas)
             .unwrap(),
         100
     );
@@ -151,7 +155,7 @@ fn fibonacci() {
 
     let contract = Contract::new(fib, code.to_vec());
 
-    let mut network = NetworkState::default();
+    let mut network = NetworkState::new();
 
     let contract_id = network.deploy(contract).unwrap();
 
@@ -162,90 +166,18 @@ fn fibonacci() {
     for i in 0..n {
         assert_eq!(
             network
-                .query::<_, u64>(contract_id, (fibonacci::COMPUTE, i), &mut gas)
+                .query::<_, u64>(
+                    contract_id,
+                    0,
+                    (fibonacci::COMPUTE, i),
+                    &mut gas
+                )
                 .unwrap(),
             fibonacci_reference(i)
         );
     }
 }
 
-struct PoseidonModule;
-
-impl Module for PoseidonModule {
-    fn id() -> ContractId {
-        ContractId::reserved(11)
-    }
-}
-
-// impl HostModule for PoseidonModule
-// where
-//     S: Store,
-// {
-//     fn execute(&self, query: Query) -> Result<ReturnValue, CanonError> {
-//         let mut source = Source::new(query.as_bytes(), &self.store);
-
-//         let qid: u8 = Canon::::read(&mut source)?;
-
-//         match qid {
-//             0 => {
-//                 let scalars: Vec<BlsScalar> = Canon::::read(&mut source)?;
-//                 let ret = dusk_poseidon::sponge::hash(&scalars);
-
-//                 ReturnValue::from_canon(&ret, &self.store)
-//             }
-//             _ => todo!(),
-//         }
-//     }
-// }
-
-// #[test]
-// fn hash_as_host_fn() {
-//     let test_inputs = [
-//         "bb67ed265bf1db490ded2e1ede55c0d14c55521509dc73f9c354e98ab76c9625",
-//         "7e74220084d75e10c89e9435d47bb5b8075991b2e29be3b84421dac3b1ee6007",
-//         "5ce5481a4d78cca03498f72761da1b9f1d2aa8fb300be39f0e4fe2534f9d4308",
-//     ];
-
-//     let test_inputs: Vec<BlsScalar> = test_inputs
-//         .iter()
-//         .map(|input| BlsScalar::from_hex_str(input).unwrap())
-//         .collect();
-
-//     let hash = HostFnTest::new();
-
-//     let store = MS::new();
-
-//     let code =
-//         include_bytes!("../target/wasm32-unknown-unknown/release/host_fn.
-// wasm");
-
-//     let contract = Contract::new(hash, code.to_vec()).unwrap();
-
-//     let mut network = NetworkState::default();
-
-//     let pos_mod = PoseidonModule::new(store.clone());
-
-//     network.register_host_module(pos_mod);
-
-//     let contract_id = network.deploy(contract).unwrap();
-
-//     let mut gas = GasMeter::with_limit(1_000_000_000);
-
-//     assert_eq!(
-//         "0xe36f4ea9b858d5c85b02770823c7c5d8253c28787d17f283ca348b906dca8528",
-//         format!(
-//             "{:#x}",
-//             network
-//                 .query::<_, BlsScalar>(
-//                     contract_id,
-//                     (host_fn::HASH, test_inputs),
-//                     &mut gas
-//                 )
-//                 .unwrap()
-//         )
-//     );
-// }
-
 #[test]
 fn block_height() {
     let bh = BlockHeight::new();
@@ -256,7 +188,7 @@ fn block_height() {
 
     let contract = Contract::new(bh, code.to_vec());
 
-    let mut network = NetworkState::with_block_height(99);
+    let mut network = NetworkState::new();
 
     let contract_id = network.deploy(contract).unwrap();
 
@@ -265,7 +197,12 @@ fn block_height() {
     assert_eq!(
         99,
         network
-            .query::<_, u64>(contract_id, block_height::BLOCK_HEIGHT, &mut gas)
+            .query::<_, u64>(
+                contract_id,
+                99,
+                block_height::BLOCK_HEIGHT,
+                &mut gas
+            )
             .unwrap()
     )
 }
@@ -280,7 +217,7 @@ fn self_snapshot() {
 
     let contract = Contract::new(bh, code.to_vec());
 
-    let mut network = NetworkState::with_block_height(99);
+    let mut network = NetworkState::new();
 
     let contract_id = network.deploy(contract).unwrap();
 
@@ -289,7 +226,7 @@ fn self_snapshot() {
     assert_eq!(
         7,
         network
-            .query::<_, i32>(contract_id, self_snapshot::CROSSOVER, &mut gas)
+            .query::<_, i32>(contract_id, 0, self_snapshot::CROSSOVER, &mut gas)
             .unwrap()
     );
 
@@ -298,6 +235,7 @@ fn self_snapshot() {
         network
             .transact::<_, i32>(
                 contract_id,
+                0,
                 (self_snapshot::SET_CROSSOVER, 9),
                 &mut gas,
             )
@@ -308,13 +246,14 @@ fn self_snapshot() {
     assert_eq!(
         9,
         network
-            .query::<_, i32>(contract_id, self_snapshot::CROSSOVER, &mut gas)
+            .query::<_, i32>(contract_id, 0, self_snapshot::CROSSOVER, &mut gas)
             .unwrap()
     );
 
     network
         .transact::<_, ()>(
             contract_id,
+            0,
             (self_snapshot::SELF_CALL_TEST_A, 10),
             &mut gas,
         )
@@ -323,12 +262,13 @@ fn self_snapshot() {
     assert_eq!(
         10,
         network
-            .query::<_, i32>(contract_id, self_snapshot::CROSSOVER, &mut gas)
+            .query::<_, i32>(contract_id, 0, self_snapshot::CROSSOVER, &mut gas)
             .unwrap()
     );
 
     let result = network.transact::<_, ()>(
         contract_id,
+        0,
         (self_snapshot::UPDATE_AND_PANIC, 11),
         &mut gas,
     );
@@ -338,7 +278,7 @@ fn self_snapshot() {
     assert_eq!(
         10,
         network
-            .query::<_, i32>(contract_id, self_snapshot::CROSSOVER, &mut gas)
+            .query::<_, i32>(contract_id, 0, self_snapshot::CROSSOVER, &mut gas)
             .unwrap()
     );
 
@@ -348,6 +288,7 @@ fn self_snapshot() {
     network
         .transact::<_, ()>(
             contract_id,
+            0,
             (self_snapshot::SELF_CALL_TEST_B, contract_id, transaction),
             &mut gas,
         )
@@ -356,7 +297,7 @@ fn self_snapshot() {
     assert_eq!(
         12,
         network
-            .query::<_, i32>(contract_id, self_snapshot::CROSSOVER, &mut gas)
+            .query::<_, i32>(contract_id, 0, self_snapshot::CROSSOVER, &mut gas)
             .unwrap()
     );
 }
@@ -370,12 +311,12 @@ fn tx_vec() {
         include_bytes!("../target/wasm32-unknown-unknown/release/tx_vec.wasm");
     let contract = Contract::new(tx_vec, code.to_vec());
 
-    let mut network = NetworkState::default();
+    let mut network = NetworkState::new();
     let contract_id = network.deploy(contract).unwrap();
     let mut gas = GasMeter::with_limit(1_000_000_000);
 
     let v = network
-        .query::<_, u8>(contract_id, tx_vec::READ_VALUE, &mut gas)
+        .query::<_, u8>(contract_id, 0, tx_vec::READ_VALUE, &mut gas)
         .unwrap();
     assert_eq!(value, v);
 
@@ -383,11 +324,11 @@ fn tx_vec() {
     let value = value + values.iter().fold(0u8, |s, v| s.wrapping_add(*v));
 
     network
-        .transact::<_, ()>(contract_id, (tx_vec::SUM, values), &mut gas)
+        .transact::<_, ()>(contract_id, 0, (tx_vec::SUM, values), &mut gas)
         .unwrap();
 
     let v = network
-        .query::<_, u8>(contract_id, tx_vec::READ_VALUE, &mut gas)
+        .query::<_, u8>(contract_id, 0, tx_vec::READ_VALUE, &mut gas)
         .unwrap();
     assert_eq!(value, v);
 
@@ -398,13 +339,14 @@ fn tx_vec() {
     network
         .transact::<_, ()>(
             contract_id,
+            0,
             (tx_vec::DELEGATE_SUM, contract_id, tx),
             &mut gas,
         )
         .unwrap();
 
     let v = network
-        .query::<_, u8>(contract_id, tx_vec::READ_VALUE, &mut gas)
+        .query::<_, u8>(contract_id, 0, tx_vec::READ_VALUE, &mut gas)
         .unwrap();
     assert_eq!(value, v);
 
@@ -415,13 +357,14 @@ fn tx_vec() {
     network
         .transact::<_, ()>(
             contract_id,
+            0,
             (tx_vec::DELEGATE_SUM, contract_id, tx),
             &mut gas,
         )
         .unwrap();
 
     let v = network
-        .query::<_, u8>(contract_id, tx_vec::READ_VALUE, &mut gas)
+        .query::<_, u8>(contract_id, 0, tx_vec::READ_VALUE, &mut gas)
         .unwrap();
     assert_eq!(value, v);
 }
@@ -441,7 +384,7 @@ fn calling() {
         "../target/wasm32-unknown-unknown/release/callee_2.wasm"
     );
 
-    let mut network = NetworkState::default();
+    let mut network = NetworkState::new();
 
     let caller_id = network
         .deploy(Contract::new(caller, code_caller.to_vec()))
@@ -458,6 +401,7 @@ fn calling() {
     network
         .transact::<_, ()>(
             caller_id,
+            0,
             (caller::SET_TARGET, callee1_id),
             &mut gas,
         )
@@ -466,6 +410,7 @@ fn calling() {
     network
         .transact::<_, ()>(
             callee1_id,
+            0,
             (caller::SET_TARGET, callee2_id),
             &mut gas,
         )
@@ -475,6 +420,7 @@ fn calling() {
         network
             .query::<_, (ContractId, ContractId, ContractId)>(
                 caller_id,
+                0,
                 caller::CALL,
                 &mut gas
             )
@@ -494,41 +440,39 @@ fn gas_consumed_host_function_works() {
 
     let contract = Contract::new(gas_contract, code.to_vec());
 
-    let mut network = NetworkState::default();
+    let mut network = NetworkState::new();
 
     let contract_id = network.deploy(contract).expect("Deploy error");
 
     // 2050 is the gas held that is known will be spent in the contract
     // after the `dusk_abi::gas_left()` call
-    let mut gas = GasMeter::with_range(2_050..1_000_000_000);
+    const CALLER_GAS_LIMIT: u64 = 1_000_000_000;
+    let mut gas = GasMeter::with_limit(CALLER_GAS_LIMIT);
 
     network
-        .transact::<_, ()>(contract_id, gas_consumed::INCREMENT, &mut gas)
+        .transact::<_, ()>(contract_id, 0, gas_consumed::INCREMENT, &mut gas)
         .expect("Transaction error");
 
     assert_eq!(
         network
-            .query::<_, i32>(contract_id, gas_consumed::VALUE, &mut gas)
+            .query::<_, i32>(contract_id, 0, gas_consumed::VALUE, &mut gas)
             .expect("Query error"),
         100
     );
 
-    let (gas_consumed, gas_left) = network
+    network
         .query::<_, (u64, u64)>(
             contract_id,
+            0,
             gas_consumed::GAS_CONSUMED,
             &mut gas,
         )
         .expect("Query error");
 
-    assert_eq!(gas_left + gas.spent(), 1_000_000_000,
-        "The gas left plus the gas spent should be equal to the initial gas provided");
-
-    assert_eq!(
-        gas.spent() - gas_consumed,
-        2_050,
-        "The gas spent minus the gas consumed should be equal to the gas held"
-    );
+    assert_eq!(gas.left() + gas.spent(), CALLER_GAS_LIMIT,
+               "The gas left plus the gas spent should be equal to the initial gas provided
+        Debug info:
+        GasMeter values: gas.left() = {}, gas.spent() = {}", gas.left(), gas.spent());
 }
 
 #[test]
@@ -540,19 +484,19 @@ fn gas_consumption_works() {
 
     let contract = Contract::new(counter, code.to_vec());
 
-    let mut network = NetworkState::default();
+    let mut network = NetworkState::new();
 
     let contract_id = network.deploy(contract).expect("Deploy error");
 
     let mut gas = GasMeter::with_limit(1_000_000_000);
 
     network
-        .transact::<_, ()>(contract_id, counter::INCREMENT, &mut gas)
+        .transact::<_, ()>(contract_id, 0, counter::INCREMENT, &mut gas)
         .expect("Transaction error");
 
     assert_eq!(
         network
-            .query::<_, i32>(contract_id, counter::READ_VALUE, &mut gas)
+            .query::<_, i32>(contract_id, 0, counter::READ_VALUE, &mut gas)
             .expect("Query error"),
         100
     );
@@ -570,14 +514,14 @@ fn out_of_gas_aborts_execution() {
 
     let contract = Contract::new(counter, code.to_vec());
 
-    let mut network = NetworkState::default();
+    let mut network = NetworkState::new();
 
     let contract_id = network.deploy(contract).expect("Deploy error");
 
     let mut gas = GasMeter::with_limit(1);
 
     let should_be_err =
-        network.transact::<_, ()>(contract_id, counter::INCREMENT, &mut gas);
+        network.transact::<_, ()>(contract_id, 0, counter::INCREMENT, &mut gas);
     assert!(format!("{:?}", should_be_err).contains("Out of Gas error"));
 
     // Ensure all gas is consumed even the tx did not succeed.
@@ -594,15 +538,69 @@ fn deploy_fails_with_floats() {
 
     let contract = Contract::new(counter, code.to_vec());
 
-    let mut network = NetworkState::default();
+    let forbidden_floats_schedule = Schedule {
+        has_forbidden_floats: false,
+        ..Default::default()
+    };
+
+    let mut network = NetworkState::with_schedule(&forbidden_floats_schedule);
 
     assert!(matches!(
         network.deploy(contract),
-        Err(rusk_vm::VMError::InstrumentalizationError(_))
+        Err(rusk_vm::VMError::InstrumentationError(_))
     ));
 }
 
-#[cfg(feature = "persistance")]
+#[test]
+fn deploy_with_id() -> Result<(), VMError> {
+    // Smallest valid WASM module possible so `deploy` won't raise a
+    // `InvalidByteCode` error
+    let code = 0x0000_0001_6D73_6100_u64.to_le_bytes();
+
+    // Create a contract with a simple state
+    let contract = Contract::new(0xfeed_u16, code.to_vec());
+
+    // Reserve a `ContractId`
+    let id = ContractId::reserved(0x10);
+
+    // Deploy with the id given
+    let mut network = NetworkState::new();
+
+    // The id is the same returned by the deploy function
+    assert_eq!(id, network.deploy_with_id(id, contract)?);
+
+    // Get the contract deployed using the same id, and verify the state is also
+    // the same
+    let state: u16 = network
+        .get_contract(&id)?
+        .state()
+        .cast()
+        .expect("Cannot cast the state");
+    assert_eq!(state, 0xfeed);
+
+    // Deploy another contract at the same address
+    let contract = Contract::new(0xcafe_u16, code.to_vec());
+    network.deploy_with_id(id, contract)?;
+
+    // Get the contract deployed using the same id, and verify the state is NOT
+    // the same as before.
+    //
+    // TODO: This means a contract CAN BE overriden once deployed, we need to
+    // decided if we should raise an error if a contract already exists with
+    // the same address
+    network.get_contract(&id)?;
+
+    let state: u16 = network
+        .get_contract(&id)?
+        .state()
+        .cast()
+        .expect("Cannot cast the state");
+    assert_eq!(state, 0xcafe);
+
+    Ok(())
+}
+
+#[cfg(feature = "persistence")]
 #[test]
 fn persistence() {
     use microkelvin::DiskBackend;
@@ -615,7 +613,7 @@ fn persistence() {
     let contract = Contract::new(counter, code.to_vec());
 
     let (persist_id, contract_id) = {
-        let mut network = NetworkState::default();
+        let mut network = NetworkState::new();
 
         let contract_id = network.deploy(contract).unwrap();
 
@@ -623,18 +621,18 @@ fn persistence() {
 
         assert_eq!(
             network
-                .query::<_, i32>(contract_id, counter::READ_VALUE, &mut gas)
+                .query::<_, i32>(contract_id, 0, counter::READ_VALUE, &mut gas)
                 .unwrap(),
             99
         );
 
         network
-            .transact::<_, ()>(contract_id, counter::INCREMENT, &mut gas)
+            .transact::<_, ()>(contract_id, 0, counter::INCREMENT, &mut gas)
             .unwrap();
 
         assert_eq!(
             network
-                .query::<_, i32>(contract_id, counter::READ_VALUE, &mut gas)
+                .query::<_, i32>(contract_id, 0, counter::READ_VALUE, &mut gas)
                 .unwrap(),
             100
         );
@@ -647,14 +645,14 @@ fn persistence() {
                         .expect("Error on tmp dir creation");
                     DiskBackend::new(dir)
                 })
-                .expect("Error in persistance"),
+                .expect("Error in persistence"),
             contract_id,
         )
     };
 
-    // If the persistance works, We should still read 100 with a freshly created
+    // If the persistence works, We should still read 100 with a freshly created
     // NetworkState.
-    let mut network = NetworkState::with_block_height(10)
+    let mut network = NetworkState::new()
         .restore(persist_id)
         .expect("Error reconstructing the NetworkState");
 
@@ -662,7 +660,7 @@ fn persistence() {
 
     assert_eq!(
         network
-            .query::<_, i32>(contract_id, counter::READ_VALUE, &mut gas)
+            .query::<_, i32>(contract_id, 0, counter::READ_VALUE, &mut gas)
             .unwrap(),
         100
     );
@@ -671,3 +669,47 @@ fn persistence() {
     std::fs::remove_dir_all(std::env::temp_dir().join("test_persist"))
         .expect("teardown fn error");
 }
+
+#[test]
+fn commit_and_reset() {
+    let counter = Counter::new(99);
+
+    let code =
+        include_bytes!("../target/wasm32-unknown-unknown/release/counter.wasm");
+
+    let contract = Contract::new(counter, code.to_vec());
+
+    let mut network = NetworkState::new();
+
+    let contract_id = network.deploy(contract).unwrap();
+    network.commit();
+
+    let mut network_clone = network.clone();
+
+    let mut gas = GasMeter::with_limit(1_000_000_000);
+
+    network
+        .transact::<_, ()>(contract_id, 0, counter::INCREMENT, &mut gas)
+        .unwrap();
+    network_clone
+        .transact::<_, ()>(contract_id, 0, counter::INCREMENT, &mut gas)
+        .unwrap();
+
+    network.commit();
+
+    network.reset();
+    network_clone.reset();
+
+    assert_eq!(
+        network
+            .query::<_, i32>(contract_id, 0, counter::READ_VALUE, &mut gas)
+            .unwrap(),
+        100
+    );
+    assert_eq!(
+        network_clone
+            .query::<_, i32>(contract_id, 0, counter::READ_VALUE, &mut gas)
+            .unwrap(),
+        99
+    );
+}
